@@ -6,6 +6,7 @@
 
 #include "quisk.h"
 #include "freedv.h"
+#include <stdint.h>
 
 int DEBUG;
 
@@ -38,19 +39,19 @@ struct freedv_advanced {	// from freedv_api.h
 #ifdef MS_WINDOWS
 #include <windows.h>
 HMODULE WINAPI hLib;
-#define GET_HANDLE1			hLib = LoadLibrary(".\\freedvpkg\\libcodec2.dll")
-#define GET_HANDLE2			hLib = LoadLibrary(".\\freedvpkg\\libcodec2_32.dll")
-#define GET_HANDLE3			hLib = LoadLibrary(".\\freedvpkg\\libcodec2_64.dll")
-#define GET_HANDLE4			hLib = LoadLibrary("libcodec2.dll")
+#define GET_HANDLE1			hLib = LoadLibrary("libcodec2.dll")
+#define GET_HANDLE2			hLib = LoadLibrary(".\\freedvpkg\\libcodec2.dll")
+#define GET_HANDLE3			hLib = LoadLibrary(".\\freedvpkg\\libcodec2_32.dll")
+#define GET_HANDLE4			hLib = LoadLibrary(".\\freedvpkg\\libcodec2_64.dll")
 #define GET_ADDR(name)		(void *)GetProcAddress(hLib, name)
 #define CLOSE_LIB			FreeLibrary(hLib)
 #else
 #include <dlfcn.h>
 void * hLib;
-#define GET_HANDLE1			hLib = dlopen("./freedvpkg/libcodec2.so", RTLD_LAZY)
-#define GET_HANDLE2			hLib = dlopen("./freedvpkg/libcodec2_32.so", RTLD_LAZY)
-#define GET_HANDLE3			hLib = dlopen("./freedvpkg/libcodec2_64.so", RTLD_LAZY)
-#define GET_HANDLE4			hLib = dlopen("libcodec2.so", RTLD_LAZY)
+#define GET_HANDLE1			hLib = dlopen("libcodec2.so", RTLD_LAZY)
+#define GET_HANDLE2			hLib = dlopen("./freedvpkg/libcodec2.so", RTLD_LAZY)
+#define GET_HANDLE3			hLib = dlopen("./freedvpkg/libcodec2_32.so", RTLD_LAZY)
+#define GET_HANDLE4			hLib = dlopen("./freedvpkg/libcodec2_64.so", RTLD_LAZY)
 #define GET_ADDR(name)		dlsym(hLib,  name)
 #define CLOSE_LIB			dlclose(hLib)
 #endif
@@ -61,8 +62,12 @@ static int quisk_freedv_squelch;
 static int interleave_frames = 1;
 static int freedv_version = -1;
 static int quisk_set_tx_bpf = 1;
+static int handle_index = -1;
+// need access to these here and in quisk.c
+int n_speech_sample_rate = 8000;
+int n_modem_sample_rate = 8000;
 
-#define SPEECH_BUF_SIZE		3000		// speech buffer size
+#define SPEECH_BUF_SIZE		12000		// speech buffer size increased from 3000
 static struct _rx_channel{
 	struct freedv * hFreedv;
 	COMP * demod_in;
@@ -70,7 +75,7 @@ static struct _rx_channel{
 	short speech_out[SPEECH_BUF_SIZE];		// output buffer
 	int speech_available;					// number of samples in output buffer
 	int playing;							// are we currently returning speech samples?
-} rx_channel[MAX_RECEIVERS] ;
+} rx_channel[MAX_RECEIVERS] = { 0 };		// safe initialisation to 0
 
 // freedv_version is the library version number, or
 //   -1		no library was found
@@ -105,11 +110,47 @@ int (*freedv_get_n_max_modem_samples)	(struct freedv *freedv);
 int (*freedv_get_n_nom_modem_samples)	(struct freedv *freedv);
 int (*freedv_get_total_bits)			(struct freedv *freedv);
 int (*freedv_get_total_bit_errors)		(struct freedv *freedv);
+int (*freedv_get_modem_sample_rate)		(struct freedv *freedv);
+int (*freedv_get_speech_sample_rate)	(struct freedv *freedv);
 //
 int (*freedv_get_sync)					(struct freedv *freedv);
 void (*freedv_set_callback_protocol)	(struct freedv *freedv, freedv_callback_protorx rx, freedv_callback_prototx tx, void *callback_state);
 void (*freedv_set_callback_data)	(struct freedv *freedv, freedv_callback_datarx datarx, freedv_callback_datatx datatx, void *callback_state);
 
+//
+// checkAvxSupport
+//
+// This function was copied from freedv, file fdmdv2_main.cpp and tweaked for use here
+//
+// Tests the underlying platform for AVX support.  2020 needs AVX support to run
+// in real-time, and old processors do not offer AVX support
+//
+
+#if defined(__x86_64__) || defined(_M_X64) || defined(__i386) || defined(_M_IX86)
+#include <cpuid.h>
+static int checkAvxSupport(void)
+{
+
+    int isAvxPresent = 0;
+    uint32_t eax, ebx, ecx, edx;
+    eax = ebx = ecx = edx = 0;
+    __cpuid(1, eax, ebx, ecx, edx);
+
+    if (ecx & (1<<27) && ecx & (1<<28)) {
+        // CPU supports XSAVE and AVX
+        uint32_t xcr0, xcr0_high;
+        asm("xgetbv" : "=a" (xcr0), "=d" (xcr0_high) : "c" (0));
+        isAvxPresent = (xcr0 & 6) == 6;    // AVX state saving enabled?
+    }
+	return isAvxPresent;
+}
+#else
+static int checkAvxSupport(void)
+{
+    return 0;
+}
+#endif
+
 /* Called when a new packet can be sent */
 void my_datatx(void *callback_state, unsigned char *packet, size_t *size) {
 	*size = 0;
@@ -117,54 +158,39 @@ void my_datatx(void *callback_state, unsigned char *packet, size_t *size) {
 
 static void GetAddrs(void)
 {
-	if (DEBUG) printf("Try handle 1\n");
+	handle_index = 1;
 	GET_HANDLE1;
-	if (hLib) {		// check the first library name
-		freedv_version = -2;
-		freedv_get_version = GET_ADDR("freedv_get_version");
-		if (freedv_get_version != NULL)
-			freedv_version = freedv_get_version();
-	}
-	if (freedv_version < 10) {		// try the next library
-		if (hLib)
-			CLOSE_LIB;
-		if (DEBUG) printf("Try handle 2\n");
+	if ( ! hLib) {
+		handle_index = 2;
 		GET_HANDLE2;
-		if (hLib) {
-			freedv_version = -2;
-			freedv_get_version = GET_ADDR("freedv_get_version");
-			if (freedv_get_version != NULL)
-				freedv_version = freedv_get_version();
+		if ( ! hLib) {
+			handle_index = 3;
+			GET_HANDLE3;
+			if ( ! hLib) {
+				handle_index = 4;
+				GET_HANDLE4;
+			}
 		}
 	}
-	if (freedv_version < 10) {		// try the next library
-		if (hLib)
-			CLOSE_LIB;
-		if (DEBUG) printf("Try handle 3\n");
-		GET_HANDLE3;
-		if (hLib) {
+	if (hLib) {
+		freedv_get_version = GET_ADDR("freedv_get_version");
+		if (freedv_get_version != NULL) {
+			freedv_version = freedv_get_version();
+			if (DEBUG)
+				printf("FreeDV codec2 library %d found, version %d\n", handle_index, freedv_version);
+		}
+		else {
 			freedv_version = -2;
-			freedv_get_version = GET_ADDR("freedv_get_version");
-			if (freedv_get_version != NULL)
-				freedv_version = freedv_get_version();
+			if (DEBUG)
+				printf("FreeDV codec2 library %d found, but no freedv_get_version() !!!\n", handle_index);
+			CLOSE_LIB;
+			return;
 		}
 	}
-	if (freedv_version < 10) {		// try the next library
-		if (hLib)
-			CLOSE_LIB;
-		if (DEBUG) printf("Try handle 4\n");
-		GET_HANDLE4;
-		if (hLib) {
-			freedv_version = -2;
-			freedv_get_version = GET_ADDR("freedv_get_version");
-			if (freedv_get_version != NULL)
-				freedv_version = freedv_get_version();
-		}
-	}
-	if (DEBUG) printf("freedv_version is %d\n", freedv_version);
-	if (freedv_version < 10) {
-		if (hLib)
-			CLOSE_LIB;
+	else {
+		handle_index = -1;
+		if (DEBUG)
+			printf("Could not find the FreeDV codec2 shared library\n");
 		return;
 	}
 
@@ -198,17 +224,24 @@ static void GetAddrs(void)
 	freedv_get_total_bits = GET_ADDR("freedv_get_total_bits");
 	freedv_get_total_bit_errors = GET_ADDR("freedv_get_total_bit_errors");
 	freedv_get_sync = GET_ADDR("freedv_get_sync");		// requires version 11
+	if( freedv_version >= 11)
+		freedv_get_modem_sample_rate = GET_ADDR("freedv_get_modem_sample_rate");		// requires version 11 or later
+	if (freedv_version >= 12) 
+		freedv_get_speech_sample_rate = GET_ADDR("freedv_get_speech_sample_rate");		// requires version 12
 	return;
 }
 
 static int quisk_freedv_rx(complex double * cSamples, double * dsamples, int count, int bank)	// Called from the sound thread.
 {	// Input digital modulation is cSamples; decoded voice is dsamples.  Each "bank" is a stream of audio.
+	// caller will have decimated input audio down to n_modem_sample_rate and reduced block size to match reduction
+	// output data will be returned at n_speech_sample_rate with corresponding block sizes
 	int i, nout, need, have, sync;
 	int n_speech_samples;
 	complex double cx;
 	double scale = (double)CLIP32 / CLIP16;	// convert 32 bits to 16 bits
 	struct freedv * hF;
 	struct _rx_channel * pCh;
+	int nCountOut = 1;	// In case we have a higher (or lower) value for o/p count
 
 	if (cSamples == NULL) {		// shutdown
 		for (i = 0; i < MAX_RECEIVERS; i++) {
@@ -227,17 +260,31 @@ static int quisk_freedv_rx(complex double * cSamples, double * dsamples, int cou
 		return 0;
 	pCh = rx_channel + bank;
 	n_speech_samples = freedv_get_n_speech_samples(hF);
+	// if (say) input modem rate is the 2020 mode rate of 8000 sa/sec
+	// and output rate is the corresponding 16000 sa/sec then for an input
+	// block size of N, outbut 2N samples block size.
+	// Apply a multiplier/divisor to 'count' for the output samples count
+	if(n_speech_sample_rate >= n_modem_sample_rate ) {
+		int nRatio = n_speech_sample_rate / n_modem_sample_rate;
+		if( nRatio >= 1 && nRatio <= 6)
+			nCountOut = nRatio * count;
+	}
+	else {
+		int nRatio = n_modem_sample_rate / n_speech_sample_rate;
+		if( nRatio >= 1 && nRatio <= 6)
+			nCountOut = count / nRatio;
+	}
 	nout = 0;
 	need = freedv_nin(hF);
 	for (i = 0; i < count; i++) {
 		cx = cRxFilterOut(cSamples[i], bank, 0);
 		if (rxMode == FDV_L)		// lower sideband
 			cx = conj(cx);
-#if 0
-		pCh->demod_in[pCh->rxdata_index].real = creal(cx) / scale;
-		pCh->demod_in[pCh->rxdata_index].imag = cimag(cx) / scale;
+#if 1 // Try it the other way and remove scaling factor
+		pCh->demod_in[pCh->rxdata_index].real = creal(cx);
+		pCh->demod_in[pCh->rxdata_index].imag = cimag(cx);
 #else
-		pCh->demod_in[pCh->rxdata_index].real = (creal(cx) - cimag(cx)) / scale;
+		pCh->demod_in[pCh->rxdata_index].real = (creal(cx) - cimag(cx));
 		pCh->demod_in[pCh->rxdata_index].imag = 0;
 #endif
 		pCh->rxdata_index++;
@@ -271,13 +318,13 @@ static int quisk_freedv_rx(complex double * cSamples, double * dsamples, int cou
 			pCh->playing = 1;
 		}
 		else {		// return zero samples
-			for (i = 0; i < count; i++)
+			for (i = 0; i < nCountOut; i++)
 				dsamples[i] = 0;
 			//if (DEBUG) printf("Rx buffer playing %d available %d\n", pCh->playing, pCh->speech_available);
-			return count;
+			return nCountOut;
 		}
 	}
-	for (nout = 0; nout < pCh->speech_available && nout < count; nout++)
+	for (nout = 0; nout < pCh->speech_available && nout < nCountOut; nout++)
 		dsamples[nout] = pCh->speech_out[nout] * scale * 0.7;
 	if (nout) {
 		pCh->speech_available -= nout;
@@ -285,7 +332,7 @@ static int quisk_freedv_rx(complex double * cSamples, double * dsamples, int cou
 	}
 	if ( ! pCh->speech_available) {
 		pCh->playing = 0;
-		while (nout < count)
+		while (nout < nCountOut)
 			dsamples[nout++] = 0;
 	}
 	//if (DEBUG) printf("Rx buffer playing %d available %d\n", pCh->playing, pCh->speech_available);
@@ -300,11 +347,16 @@ static int quisk_freedv_tx(complex double * cSamples, double * dsamples, int cou
 	static COMP * mod_out = NULL;
 	static short * speech_in = NULL;
 	static int speech_index=0, mod_index=0;
+	// g8kbb - add for 800XA
+	static short *real_mod_out = NULL;
 
 	if (dsamples == NULL) {		// shutdown
 		if (mod_out)
 			free(mod_out);
 		mod_out = NULL;
+		if (real_mod_out)
+			free(real_mod_out);
+		real_mod_out = NULL;
 		if (speech_in)
 			free(speech_in);
 		speech_in = NULL;
@@ -314,28 +366,51 @@ static int quisk_freedv_tx(complex double * cSamples, double * dsamples, int cou
 		return 0;
 	n_speech_samples = freedv_get_n_speech_samples(rx_channel[0].hFreedv);
 	n_nom_modem_samples = freedv_get_n_nom_modem_samples(rx_channel[0].hFreedv);
+	// find if need to output at faster rate
+	int nRatio = n_modem_sample_rate / n_speech_sample_rate;
+	if( nRatio < 1 || nRatio > 6 )
+		nRatio = 1;
 	if (mod_out == NULL) {		// initialize
 		mod_out = (COMP *)malloc(sizeof(COMP) * n_nom_modem_samples);
 		memset(mod_out, 0, sizeof(COMP) * n_nom_modem_samples);
 		speech_in = (short*)malloc(sizeof(short) * n_speech_samples);
 		speech_index=0;
 		mod_index=0;
+		real_mod_out = (short *)malloc(sizeof(short) * n_nom_modem_samples);
+		memset(real_mod_out, 0, sizeof(short) * n_nom_modem_samples);
+		//if( DEBUG ) printf("Initialise tx buffer size = %d, i/p sample size = %d\n", n_nom_modem_samples, n_speech_samples );
 	}
 	nout = 0;
 	for (i = 0; i < count; i++) {
 		speech_in[speech_index++] = (short)dsamples[i];
 		if (speech_index >= n_speech_samples) {
 			// Calculate a new block, but first write out the rest of the old block
-			for ( ; mod_index < n_nom_modem_samples; mod_index++)
-				cSamples[nout++] = mod_out[mod_index].real + I * mod_out[mod_index].imag;
-			freedv_comptx(rx_channel[0].hFreedv, mod_out, speech_in);
+			// 800xa uses real not complex
+			if( freedv_current_mode == FREEDV_MODE_800XA ) {
+				for ( ; mod_index < n_nom_modem_samples; mod_index++)
+					cSamples[nout++] = real_mod_out[mod_index];
+				freedv_tx(rx_channel[0].hFreedv, real_mod_out, speech_in);
+			}
+			else {
+				for ( ; mod_index < n_nom_modem_samples; mod_index++)
+					cSamples[nout++] = mod_out[mod_index].real + I * mod_out[mod_index].imag;
+				freedv_comptx(rx_channel[0].hFreedv, mod_out, speech_in);
+			}
+			//if( DEBUG ) printf("Block processed, speech index=%d, n_speech_samples=%d, count=%d\n", speech_index, n_speech_samples, count );
 			mod_index = 0;
 			speech_index = 0;
 		}
-		else {		// write out samples slowly
-			if (mod_index < n_nom_modem_samples) {
-				cSamples[nout++] = mod_out[mod_index].real + I * mod_out[mod_index].imag;
-				mod_index++;
+		else {		// write out samples
+			for( int j=0; j<nRatio; j++) {
+				if (mod_index < n_nom_modem_samples ) {
+					if( freedv_current_mode == FREEDV_MODE_800XA ) {
+						cSamples[nout++] = real_mod_out[mod_index];
+					}
+					else {
+						cSamples[nout++] = mod_out[mod_index].real + I * mod_out[mod_index].imag;
+					}
+					mod_index++;
+				}
 			}
 		}
 	}
@@ -396,10 +471,12 @@ static void CloseFreedv(void)	// Called from the GUI thread or sound thread
 			freedv_close(rx_channel[i].hFreedv);
 			rx_channel[i].hFreedv = NULL;
 		}
+#if 0 // g8kbb - unnecessary - quisk_freedv_rx() will do it
 		if (rx_channel[i].demod_in) {
 			free(rx_channel[i].demod_in);
 			rx_channel[i].demod_in = NULL;
 		}
+#endif
 	}
 	quisk_freedv_rx(NULL, NULL, 0, 0);
 	quisk_freedv_tx(NULL, NULL, 0);
@@ -413,17 +490,52 @@ static int OpenFreedv(void)	// Called from the GUI thread or sound thread
 
 	if ( ! hLib)
 		GetAddrs();		// Get the entry points for funtions in the codec2 library
-	if (DEBUG) printf("freedv_open: version %d\n", freedv_version);
+	if (DEBUG) {	
+		switch (handle_index) {
+		case 1:
+			printf("freedv_open: system codec2 library found, version %d\n", freedv_version);
+			break;
+		case 2:
+			printf("freedv_open: codec2 library freedvpkg/libcodec2.dll|so found, version %d\n", freedv_version);
+			break;
+		case 3:
+		case 4:
+			printf("freedv_open: codec2 library freedvpkg/libcodec2_32|64.dll|so found, version %d\n", freedv_version);
+			break;
+		default:
+			printf("freedv_open: Could not find the FreeDV codec2 shared library\n");
+			break;
+		}
+	}
 	if (freedv_version < 10) {
 		CloseFreedv();
 		requested_mode = -1;
+		if (DEBUG)
+			printf("freedv_open: Failure because version is less than 10\n");
 		return 0;	// failure
 	}
-    	if (requested_mode == FREEDV_MODE_700D && freedv_open_advanced) {
-        	struct freedv_advanced adv;
-        	adv.interleave_frames = interleave_frames;
-        	hF = freedv_open_advanced(requested_mode, &adv);
-    	}
+	if (requested_mode == FREEDV_MODE_2020) {
+		if (checkAvxSupport() == 0) {
+			CloseFreedv();
+			requested_mode = -1;
+			if (DEBUG)
+				printf("freedv_open: Failure because mode 2020 requires Avx support\n");
+			return 0;	// failure
+		}
+		if (handle_index >= 3) {	// Quisk provided codec2
+			CloseFreedv();
+			requested_mode = -1;
+			if (DEBUG)
+				printf("freedv_open: Failure because mode 2020 requires a system installation of codec2\n");
+			return 0;	// failure
+		}
+	}
+
+   	if (requested_mode == FREEDV_MODE_700D && freedv_open_advanced) {
+       	struct freedv_advanced adv;
+       	adv.interleave_frames = interleave_frames;
+       	hF = freedv_open_advanced(requested_mode, &adv);
+   	}
 	else {
         	hF = freedv_open(requested_mode);
 	}
@@ -446,6 +558,12 @@ static int OpenFreedv(void)	// Called from the GUI thread or sound thread
 	if (freedv_set_tx_bpf)
 		freedv_set_tx_bpf(hF, quisk_set_tx_bpf);
 	n_max_modem_samples = freedv_get_n_max_modem_samples(hF);
+	n_speech_sample_rate = 8000;
+	if (freedv_version >= 12 && freedv_get_speech_sample_rate)
+		n_speech_sample_rate = freedv_get_speech_sample_rate(hF);
+	n_modem_sample_rate = 8000;
+	if (freedv_version >= 11 && freedv_get_modem_sample_rate)
+		n_modem_sample_rate = freedv_get_modem_sample_rate(hF);
 	for (i = 0; i < MAX_RECEIVERS; i++) {
 		rx_channel[i].rxdata_index = 0;
 		rx_channel[i].speech_available = 0;
@@ -453,6 +571,11 @@ static int OpenFreedv(void)	// Called from the GUI thread or sound thread
 		if (rx_channel[i].demod_in)
 			free(rx_channel[i].demod_in);
 		rx_channel[i].demod_in = (COMP *)malloc(sizeof(COMP) * n_max_modem_samples);
+		if( rx_channel[i].demod_in == NULL) {
+			CloseFreedv();
+			requested_mode = -1;
+			return 0; // failure
+		}
 		if (i > 0) {
 			rx_channel[i].hFreedv = freedv_open(requested_mode);
 			if (rx_channel[i].hFreedv)
@@ -462,6 +585,10 @@ static int OpenFreedv(void)	// Called from the GUI thread or sound thread
 	if (DEBUG) printf("n_nom_modem_samples %d\n", freedv_get_n_nom_modem_samples(rx_channel[0].hFreedv));
 	if (DEBUG) printf("n_speech_samples %d\n", freedv_get_n_speech_samples(rx_channel[0].hFreedv));
 	if (DEBUG) printf("n_max_modem_samples %d\n", n_max_modem_samples);
+	if (DEBUG) {
+	 	if (freedv_version >= 11) printf("modem_sample_rate %d\n", n_modem_sample_rate);
+		if (freedv_version >= 12) printf("speech_sample_rate %d\n", n_speech_sample_rate);
+	}
 	freedv_current_mode = requested_mode;
 	return 1;		// success
 }
@@ -500,7 +627,6 @@ PyObject * quisk_freedv_set_options(PyObject * self, PyObject * args, PyObject *
 	int bpf=-1;
 	char * ptMsg=NULL;
 	static char * kwlist[] = {"mode", "tx_msg", "DEBUG", "squelch", "interleave_frames", "set_tx_bpf", NULL} ;
-	struct freedv * hFreedv;
 
 	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|isiiii", kwlist, &mode, &ptMsg, &DEBUG, &quisk_freedv_squelch, &interleave_frames, &bpf))
 		return NULL;
@@ -511,20 +637,23 @@ PyObject * quisk_freedv_set_options(PyObject * self, PyObject * args, PyObject *
 		if (freedv_set_tx_bpf && rx_channel[0].hFreedv)
 			freedv_set_tx_bpf(rx_channel[0].hFreedv, quisk_set_tx_bpf);
 	}
-	if (mode == -1)
+	if (mode == -1) {	// mode was not entered
 		;
-	else if (freedv_current_mode < 0)		// not started
-		requested_mode = mode;
-	else if (freedv_version == 10 && mode == 0)
-		requested_mode = mode;
-	else if (freedv_version == 11 && mode <= 2)
-		requested_mode = mode;
-	else {
-		hFreedv = freedv_open(mode);	// test new mode
-		if (hFreedv != NULL) {
-			freedv_close(hFreedv);
+	}
+	else if (mode == FREEDV_MODE_2020) {
+		if (checkAvxSupport() && handle_index <= 2 && freedv_version >= 13)
 			requested_mode = mode;
-		}
+	}
+	else if (freedv_version == 10) {
+		if (mode == 0)
+			requested_mode = mode;
+	}
+	else if (freedv_version == 11) {
+		if (mode <= 2)
+			requested_mode = mode;
+	}
+	else {
+		requested_mode = mode;
 	}
 	return PyInt_FromLong(requested_mode);	// Return the mode
 }
@@ -548,3 +677,19 @@ PyObject * quisk_freedv_get_version(PyObject * self, PyObject * args)	// Called
 		GetAddrs();		// Get the entry points for funtions in the codec2 library
 	return PyInt_FromLong(freedv_version);
 }
+
+PyObject * quisk_freedv_set_squelch_en(PyObject * self, PyObject * args)	// Called from the GUI thread.
+{
+	int i,state;
+
+	if (!PyArg_ParseTuple (args, "i", &state))
+		return NULL;
+	quisk_freedv_squelch = state;
+	for (i = 0; i < MAX_RECEIVERS; i++) {
+		if( rx_channel[i].hFreedv )
+			freedv_set_squelch_en(rx_channel[i].hFreedv, state);
+	}
+	return PyInt_FromLong(state);
+}
+
+
