@@ -64,13 +64,9 @@ class Hardware(BaseHardware):
     self.pc2hermes[4 * 9] = 63	# C0 index == 0b1001, C1[7:0] Tx level
     for c0 in range(1, 9):		# Set all frequencies to 7012352, 0x006B0000
       self.SetControlByte(c0, 2, 0x6B)
-    value = conf.keyupDelay
-    if value > 1023:
-      value = 1023
-    self.SetControlByte(0x10, 2, value & 0x3)		# cw_hang_time
-    self.SetControlByte(0x10, 1, (value >> 2) & 0xFF)	# cw_hang_time
     self.SetLowPwrEnable(conf.hermes_lowpwr_tr_enable)
     self.EnablePowerAmp(conf.hermes_power_amp)
+    self.ChangeTxLNA(conf.hermes_TxLNA_dB)
     self.MakePowerCalibration()
   def pre_open(self):
     # This socket is used for the Metis Discover protocol
@@ -390,13 +386,16 @@ class Hardware(BaseHardware):
     # mode is a string: "USB", "AM", etc.
     BaseHardware.ChangeMode(self, mode)
     self.mode = mode
-    self.SetTxLevel()
-  def OnButtonPTT(self, event):
-    btn = event.GetEventObject()
-    if btn.GetValue():
-      QS.set_PTT(1)
+    if mode == "CWL" or mode == "CWU":
+      value = self.conf.keyupDelay
     else:
-      QS.set_PTT(0)
+      value = 50
+    if value > 1023:
+      value = 1023
+    self.SetControlByte(0x10, 2, value & 0x3)		# cw_hang_time
+    self.SetControlByte(0x10, 1, (value >> 2) & 0xFF)	# cw_hang_time
+    if DEBUG: print ("Change keyup delay to", value)
+    self.SetTxLevel()
   def OnSpot(self, level):
     # level is -1 for Spot button Off; else the Spot level 0 to 1000.
     pass
@@ -437,9 +436,9 @@ class Hardware(BaseHardware):
     QS.pc_to_hermes(self.pc2hermes)
     if DEBUG: print ("Change AGC to", value)
   ## Simpler LNA setting for HL2 identifying as version >=40, see HL2 wiki for details
-  def ChangeLNA(self, value):
+  def ChangeLNA(self, value):		# LNA for Rx
     # value is -12 to +48
-    if self.hermes_code_version < 40:
+    if self.hermes_code_version < 40:	# Is this correct ??
       if value < 20:
         self.pc2hermes[2] |= 0x08			# C0 index == 0, C3[3]: LNA +32 dB disable == 1
         value = 19 - value
@@ -451,6 +450,12 @@ class Hardware(BaseHardware):
     self.pc2hermes[4 * 10 + 3] = value			# C0 index == 0x1010, C4[4:0] LNA 0-32 dB gain
     QS.pc_to_hermes(self.pc2hermes)
     if DEBUG: print ("Change LNA to", value)
+  def ChangeTxLNA(self, value):		# LNA for Tx
+    # value is -12 to +48
+    value = ((value+12) & 0x3f) | 0x40 | 0x80
+    self.SetControlByte(0x0e, 3, value)		# C0 index == 0x0e, C3
+    QS.pc_to_hermes(self.pc2hermes)
+    if DEBUG: print ("Change Tx LNA to", value)
   def SetTxLevel(self):
     try:
       tx_level = self.conf.tx_level[self.band]
