@@ -4,7 +4,7 @@
 
 // Sound parameters
 //
-#define QUISK_SC_SIZE		128
+#define QUISK_SC_SIZE		256
 #define QUISK_PATH_SIZE     256         // max file path length
 #define IP_SIZE				32
 #define MAX_FILTER_SIZE		10001
@@ -66,16 +66,47 @@ typedef enum _rx_mode {
 /*!
  * \brief Specifies which driver a \c sound_dev is opened with
  */
+
+typedef enum {
+	Int16,
+	Int24,
+	Int32,
+	Float32
+} sound_format_t;
+
+typedef enum {
+	t_Capture,
+	t_Playback,
+	t_MicCapture,
+	t_MicPlayback,
+	t_DigitalInput,
+	t_DigitalOutput,
+	t_RawSamplePlayback,
+	t_DigitalRx1Output
+} device_index_t;
+
 typedef enum dev_driver{
    DEV_DRIVER_NONE = 0,
    DEV_DRIVER_PORTAUDIO,
    DEV_DRIVER_ALSA,
-   DEV_DRIVER_PULSEAUDIO
+   DEV_DRIVER_PULSEAUDIO,
+   DEV_DRIVER_DIRECTX,
+   DEV_DRIVER_SOUNDIO
 } dev_driver_t;
 
+typedef enum {
+	STARTING,
+	RECEIVE,
+	HARDWARE_CWKEY,
+	HARDWARE_PTT,
+	SOFTWARE_CWKEY,
+	SOFTWARE_PTT,
+	} play_state_t;
+
 struct sound_dev {				// data for sound capture or playback device
-	char name[QUISK_SC_SIZE];	// string name of device
+	char name[QUISK_SC_SIZE];		// the name of the device for display
 	char stream_description[QUISK_SC_SIZE]; // Short description of device/stream
+	char device_name[QUISK_SC_SIZE];	// hardware device name 
 	void * handle;				// Handle of open device, or NULL
 	dev_driver_t driver;		// Which audio driver the device is using
 	void * buffer;				// Handle of buffer for device
@@ -97,7 +128,7 @@ struct sound_dev {				// data for sound capture or playback device
    // frames.
 	int read_frames;
 	int latency_frames;			// desired latency in audio play samples
-	int play_buf_size;			// size of playback buffer in samples
+	int play_buf_size;			// size of the sound card playback buffer in frames or bytes
 	int use_float;				// DirectX: Use IEEE floating point
 	int dataPos;				// DirectX: data position
 	int oldPlayPos;				// DirectX: previous value of playPos
@@ -110,15 +141,19 @@ struct sound_dev {				// data for sound capture or playback device
 	unsigned int rate_max;
 	unsigned int chan_min;		// min and max available number of channels
 	unsigned int chan_max;
-	complex double dc_remove;			// filter to remove DC from samples
-	double save_sample;			// Used to delay the I or Q sample
+	complex double dc_remove;	// filter to remove DC from samples
+	double save_sample;		// Used to delay the I or Q sample
 	char msg1[QUISK_SC_SIZE];	// string for information message
+	char dev_errmsg[QUISK_SC_SIZE];	// error message for device, or ""
 	int stream_dir_record;		// 1 for recording, 0 for playback
 	char server[IP_SIZE];		// server string for remote pulseaudio
-	int stream_format;			// format of pulseaudio device
+	int stream_format;		// format of pulseaudio device
 	int pulse_stream_state;		// state of the pulseaudio stream
 	volatile int cork_status;	// 1 for corked, 0 for uncorked
 	double average_square;		// average of squared sample magnitude
+	sound_format_t sound_format;	// format of sound array for the sound device
+	device_index_t dev_index;	// identify devices
+	void * device_data;		// special data for each sound device
 } ;
 
 struct sound_conf {
@@ -153,7 +188,8 @@ struct sound_conf {
 	int mic_channel_Q;
 	double mic_out_volume;
 	char IQ_server[IP_SIZE];	//IP address of optional streaming IQ server (pulseaudio)
-	int verbose_pulse;      //verbose output for pulse audio
+	int verbose_pulse;      // verbose output for pulse audio
+	int verbose_sound;	// verbose output for other sound systems
 } ;
 
 enum quisk_rec_state {
@@ -165,6 +201,13 @@ enum quisk_rec_state {
 	PLAY_SAMPLES } ;
 extern enum quisk_rec_state quisk_record_state;
 
+struct wav_file {
+	FILE * fp;
+	char file_name[QUISK_PATH_SIZE];
+	int enable;
+	unsigned long samples;
+};
+
 struct QuiskWav {			// data to create a WAV or RAW audio file
     double scale;
     int sample_rate;
@@ -182,10 +225,12 @@ void QuiskWavClose(struct QuiskWav *);
 int QuiskWavWriteOpen(struct QuiskWav *, char *, short, short, short, int, double);
 void QuiskWavWriteC(struct QuiskWav *, complex double *, int);
 void QuiskWavWriteD(struct QuiskWav *, double *, int);
+void QuiskWavWriteF(struct QuiskWav *, float *, int);
 int QuiskWavReadOpen(struct QuiskWav *, char *, short, short, short, int, double);
 void QuiskWavReadC(struct QuiskWav *, complex double *, int);
 void QuiskWavReadD(struct QuiskWav *, double *, int);
-void QuiskMeasureRate(const char *, int);
+void QuiskMeasureRate(const char *, int, int);
+void quisk_record_audio(struct wav_file *, complex double *, int);
 
 extern struct sound_conf quisk_sound_state, * pt_quisk_sound_state;
 extern int mic_max_display;		// display value of maximum microphone signal level
@@ -216,6 +261,18 @@ extern complex double cRxFilterOut(complex double, int, int);
 extern int quisk_multirx_count;			// number of additional receivers zero or 1, 2, 3, ..
 extern struct sound_dev quisk_DigitalRx1Output;		// Output sound device for sub-receiver 1
 extern int quisk_is_vna;			// is this the VNA program?
+extern int quisk_serial_key_errors;		// Error count for the Quisk internal serial key
+extern double quisk_sidetoneVolume;		// Audio output level of the CW sidetone, 0.0 to 1.0
+extern int quisk_serial_key_down;		// The state of the serial port CW key
+extern int quisk_hardware_cwkey;		// The state of the hardware CW key from UDP or USB
+extern int quisk_sidetoneFreq;			// Frequency in hertz for the sidetone
+extern int quisk_active_sidetone;		// Whether and how to generate a sidetone
+extern int quisk_isFDX;				// Are we in full duplex mode?
+extern int quisk_use_serial_port;		// Are we using the serial port for CW key or PTT?
+extern play_state_t quisk_play_state;		// startup, receiving, sidetone
+extern int freedv_current_mode;			// current FreeDV mode; 700D, @)@) etc.
+extern int n_modem_sample_rate;			// Receive data, decimate to modem_sample_rate, FreeDV codec output data at speech_sample_rate
+extern int n_speech_sample_rate;		// Microphone decimate to speech_sample_rate, Freedv codec output data at modem_sample_rate, interpolate to 48000
 
 extern PyObject * quisk_set_spot_level(PyObject * , PyObject *);
 extern PyObject * quisk_get_tx_filter(PyObject * , PyObject *);
@@ -226,6 +283,7 @@ extern PyObject * quisk_play_channels(PyObject * , PyObject *);
 extern PyObject * quisk_micplay_channels(PyObject * , PyObject *);
 extern PyObject * quisk_sound_devices(PyObject * , PyObject *);
 extern PyObject * quisk_pa_sound_devices(PyObject * , PyObject *);
+extern PyObject * quisk_sio_sound_devices(PyObject * , PyObject *);
 extern PyObject * quisk_sound_errors(PyObject *, PyObject *);
 extern PyObject * quisk_set_file_record(PyObject *, PyObject *);
 extern PyObject * quisk_set_file_name(PyObject *, PyObject *, PyObject *);
@@ -243,6 +301,10 @@ extern PyObject * quisk_freedv_get_version(PyObject *, PyObject *);
 extern PyObject * quisk_freedv_get_rx_char(PyObject *, PyObject *);
 extern PyObject * quisk_freedv_set_options(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_set_sparams(PyObject *, PyObject *, PyObject *);
+extern PyObject * quisk_freedv_set_squelch_en(PyObject *, PyObject *);
+extern PyObject * quisk_open_key(PyObject *, PyObject *);
+extern PyObject * quisk_close_key(PyObject *, PyObject *);
+extern PyObject * quisk_set_sound_name(PyObject *, PyObject *);
 
 // These function pointers are the Start/Stop/Read interface for
 // the SDR-IQ and any other C-language extension modules that return
@@ -265,8 +327,6 @@ int quisk_read_sound(void);
 int quisk_process_microphone(int, complex double *, int);
 void quisk_open_mic(void);
 void quisk_close_mic(void);
-int quisk_open_key(const char *);
-void quisk_close_key(void);
 void quisk_set_key_down(int);
 void quisk_set_tx_mode(void);
 void ptimer(int);
@@ -281,6 +341,10 @@ void quisk_udp_mic_error(char *);
 void quisk_check_freedv_mode(void);
 void quisk_calc_audio_graph(double, complex double *, double *, int, int);
 int QuiskDeltaMsec(int);
+void * quisk_make_sidetone(struct sound_dev *, int);
+void quisk_set_play_state(int);
+void quisk_poll_hardware_key(void);
+void quisk_set_sidetone(void);
 
 // Functions supporting digital voice codecs
 typedef int  (* ty_dvoice_codec_rx)(complex double *, double *, int, int);
@@ -291,6 +355,7 @@ extern ty_dvoice_codec_tx  pt_quisk_freedv_tx;
 // Driver function definitions=================================================
 int  quisk_read_alsa(struct sound_dev *, complex double *);
 void quisk_play_alsa(struct sound_dev *, int, complex double *, int, double);
+void quisk_alsa_sidetone(struct sound_dev *);
 void quisk_start_sound_alsa(struct sound_dev **, struct sound_dev **);
 void quisk_close_sound_alsa(struct sound_dev **, struct sound_dev **);
 
@@ -307,6 +372,11 @@ void quisk_start_sound_pulseaudio(struct sound_dev **, struct sound_dev **);
 void quisk_close_sound_pulseaudio(void);
 void quisk_cork_pulseaudio(struct sound_dev *, int);
 void quisk_flush_pulseaudio(struct sound_dev *);
+
+int  quisk_read_soundio(struct sound_dev *, complex double *);
+void quisk_write_soundio(struct sound_dev *, int, complex double *);
+void quisk_start_sound_soundio(struct sound_dev **, struct sound_dev **);
+void quisk_close_sound_soundio(struct sound_dev **, struct sound_dev **);
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 /*
