@@ -982,8 +982,9 @@ class App(wx.App):
       h = 0
     QS.set_enable_bandscope(0)
     # FFT size must equal the data_width so that all data points are returned!
+    wisdom_path = os.path.join(os.path.dirname(self.local_conf.StatePath), 'quisk_wisdom.cache')
     QS.record_app(self, conf, self.data_width, self.data_width, self.data_width,
-                 1, self.sample_rate, h)
+                 1, self.sample_rate, h, wisdom_path)
     # Make all the screens and hide all but one
     self.graph = GraphScreen(frame, self.data_width, self.data_width, self.correct_width, self.correct_delta)
     self.screen = self.graph
@@ -1119,10 +1120,11 @@ class App(wx.App):
       self.config_text = "Missing config_text"
     # Note: Subsequent calls to set channels must not name a higher channel number.
     #       Normally, these calls are only used to reverse the channels.
-    QS.open_sound(conf.name_of_sound_capt, '', self.sample_rate,
-                conf.data_poll_usec, conf.latency_millisecs,
-                '', conf.tx_ip, conf.tx_audio_port,
-                48000, 0, 0, 1.0, '', 48000)
+    index = 0
+    QS.open_sound(0, conf.data_poll_usec, conf.latency_millisecs,
+                conf.tx_ip, conf.tx_audio_port,
+                conf.mic_sample_rate, conf.mic_channel_I, conf.mic_channel_Q,
+				conf.mic_out_volume, conf.mic_playback_rate)
     self.Bind(wx.EVT_IDLE, self.graph.OnIdle)
     frame.Show()
     self.NewFreq(1000000, 30000000)
