@@ -31,6 +31,7 @@
 /*
  * 2017 Nov by N2ADR Remove most exit(1) statements.
  * 2017 Nov by N2ADR Add code to enable PulseAudio to continue with mis-spelled device names.
+ * 2020 Apr by N2ADR Changes in printf's
 */
 
 #include <Python.h>
@@ -77,44 +78,47 @@ void stream_state_callback(pa_stream *s, void *userdata) {
     switch (dev->pulse_stream_state) {
         case PA_STREAM_CREATING:
 	    if (quisk_sound_state.verbose_pulse)
-		printf("\n**Stream %s state Creating\n", dev->name);
+		printf("\n**Stream state Creating: %s; %s\n", dev->stream_description, dev->name);
             break;
             
         case PA_STREAM_TERMINATED:
 	    if (quisk_sound_state.verbose_pulse)
-		printf("\n**Stream %s state Terminated\n", dev->name);
+		printf("\n**Stream state Terminated: %s; %s\n", dev->stream_description, dev->name);
             streams_ready--;
             break;
             
         case PA_STREAM_READY:
 	    if (quisk_sound_state.verbose_pulse)
-		printf("\n**Stream %s state Ready\n", dev->name);
+		printf("\n**Stream state Ready: %s; %s\n", dev->stream_description, dev->name);
             streams_ready++; //increment counter to tell other thread that this stream is ready
 	    streams_to_start++;
             if (quisk_sound_state.verbose_pulse) {
                 const pa_buffer_attr *a;
-                printf("Connected to device %s (%u, %ssuspended). ",
-                       pa_stream_get_device_name(s), pa_stream_get_device_index(s),
-                       pa_stream_is_suspended(s) ? "" : "not ");
+                printf("   Connected to device index %u, %ssuspended: %s.\n",
+			pa_stream_get_device_index(s),
+                       pa_stream_is_suspended(s) ? "" : "not ",
+                       pa_stream_get_device_name(s));
 
                 if (!(a = pa_stream_get_buffer_attr(s)))
-                    printf("pa_stream_get_buffer_attr() failed: %s", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
+                    printf("   pa_stream_get_buffer_attr() failed: %s", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
                 else if (!(a->prebuf)) {
-                    printf("Buffer metrics %s: maxlength=%u, fragsize=%u\n", dev->name, a->maxlength, a->fragsize);
+                    printf("   Buffer metrics: maxlength=%u, fragsize=%u\n", a->maxlength, a->fragsize);
                 }
                 else {
-                    printf("Buffer metrics %s: maxlength=%u, prebuf=%u, tlength=%u  minreq=%u\n",
-                           dev->name, a->maxlength, a->prebuf, a->tlength, a->minreq);
+                    printf("   Buffer metrics: maxlength=%u, prebuf=%u, tlength=%u  minreq=%u\n",
+                           a->maxlength, a->prebuf, a->tlength, a->minreq);
                 }
             }
             break;
         
         case PA_STREAM_FAILED:
         default:
+	    snprintf(dev->dev_errmsg, QUISK_SC_SIZE,
+		"%.60s: %.60s", dev->device_name, pa_strerror(pa_context_errno(pa_stream_get_context(s))));
 	    snprintf(quisk_sound_state.err_msg, QUISK_SC_SIZE,
 		"Stream error: %.40s - %.40s", dev->name, pa_strerror(pa_context_errno(pa_stream_get_context(s))));
 	    if (quisk_sound_state.verbose_pulse)
-		printf("\n**Stream %s state Failed\n", dev->name);
+		printf("\n**Stream state Failed: %s; %s\n", dev->stream_description, dev->device_name);
             printf("%s\n", quisk_sound_state.err_msg);
 	    streams_to_start++;
             break;
@@ -129,7 +133,7 @@ static void stream_underflow_callback(pa_stream *s, void *userdata) {
     assert(dev);
     
     if (quisk_sound_state.verbose_pulse)
-        printf("Stream underrun %s\n", dev->name);
+        printf("Stream underrun %s\n", dev->stream_description);
     dev->dev_error++;
 }
 
@@ -139,7 +143,7 @@ static void stream_overflow_callback(pa_stream *s, void *userdata) {
     assert(s);
     
     if (quisk_sound_state.verbose_pulse)
-        printf("Stream overrun %s\n", dev->name);
+        printf("Stream overrun %s\n", dev->stream_description);
     dev->dev_error++;
 }
 
@@ -150,7 +154,7 @@ static void stream_started_callback(pa_stream *s, void *userdata) {
     assert(dev);
     
     if (quisk_sound_state.verbose_pulse)
-        printf("Stream started %s\n", dev->name);
+        printf("Stream started %s\n", dev->stream_description);
 }
 
 //Called when cork/uncork has succeeded on passed stream. Signals mainloop when complete.
@@ -160,12 +164,12 @@ static void stream_corked_callback(pa_stream *s, int success, void *userdata) {
     
     if (s) {
         if (quisk_sound_state.verbose_pulse)
-            printf("Stream cork/uncork %s success\n", dev->name);
+            printf("Stream cork/uncork %s success\n", dev->stream_description);
         pa_threaded_mainloop_signal(pa_ml, 0);
     }
     else {
         if (quisk_sound_state.verbose_pulse)
-            printf("Stream cork/uncork %s Failure!\n", dev->name);
+            printf("Stream cork/uncork %s Failure!\n", dev->stream_description);
         exit(11);
     }
 }
@@ -176,11 +180,11 @@ static void stream_flushed_callback(pa_stream *s, int success, void *userdata) {
     struct sound_dev *dev = userdata;
     
     if (s) {
-        printf("Stream flush %s success\n", dev->name);
+        printf("Stream flush %s success\n", dev->stream_description);
         pa_threaded_mainloop_signal(pa_ml, 0);
     }
     else {
-        printf("Stream flush %s Failure!\n", dev->name);
+        printf("Stream flush %s Failure!\n", dev->stream_description);
         exit(12);
     }
 }
@@ -228,13 +232,12 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
         pa_zero(rec_ba);
         pa_zero(play_ba);
 
-        if (dev->name[5] == ':')
-            dev_name = dev->name + 6;		// the device name is given; "pulse:alsa_input.pci-0000_00_1b.0.analog-stereo"
-        else
-            dev_name = NULL;			// the device name is "pulse" for the default device
+	dev_name = dev->device_name;
+	if (strcmp(dev_name, "default") == 0)		// the device name is "default" for the default device
+		dev_name = NULL;
 
         if (quisk_sound_state.verbose_pulse)
-            printf("Opening Device %s\n", dev_name);
+            printf("Opening Device %s", dev_name);
 
         //Construct sample specification. Use S16LE if availiable. Default to Float32 for others.
         //If the source/sink is not Float32, Pulseaudio will convert it (uses CPU)
@@ -242,10 +245,16 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
         if (default_ss.format == PA_SAMPLE_S16LE) {
             dev->sample_bytes = 2;
             ss.format = default_ss.format;
+            dev->sound_format = Int16;
+            if (quisk_sound_state.verbose_pulse)
+                printf(" with sound format Int16\n");
         }
         else {
             dev->sample_bytes = 4;
             ss.format = PA_SAMPLE_FLOAT32LE;
+            dev->sound_format = Float32;
+            if (quisk_sound_state.verbose_pulse)
+                printf(" with sound format Float32\n");
         }
         
         ss.rate = dev->sample_rate;
@@ -253,7 +262,6 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
 
         rec_ba.maxlength = (uint32_t) -1;
         rec_ba.fragsize = (uint32_t) SAMP_BUFFER_SIZE / 16;  //higher numbers eat cpu on reading monitor streams.
-
         play_ba.maxlength = (uint32_t) -1;
         play_ba.prebuf = (uint32_t) (dev->sample_bytes * ss.channels * dev->latency_frames);
         //play_ba.tlength = (uint32_t) -1;
@@ -263,7 +271,6 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
             play_ba.minreq = (uint32_t) 0; //verify this is sane
         else
             play_ba.minreq = (uint32_t) -1;
-
         if (dev->stream_dir_record) {
 
             if (!(s = pa_stream_new(c, dev->stream_description, &ss, NULL))) {
@@ -410,7 +417,6 @@ void quisk_flush_pulseaudio(struct sound_dev *dev) {
     pa_threaded_mainloop_unlock(pa_ml);
 }
 
-
 static void WaitForPoll(void) {		// Implement a blocking read
     static double time0 = 0;		// start time in seconds
     double timer;			// time remaining from last poll usec
@@ -555,7 +561,7 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
     int i=0, n=0;
     void *fbuffer;
     int fbuffer_bytes = 0;
-    
+
     if( !dev || nSamples <= 0 || dev->pulse_stream_state != PA_STREAM_READY)
         return;
     
@@ -616,21 +622,19 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
     pa_threaded_mainloop_lock(pa_ml);
     size_t writable = pa_stream_writable_size(s);
 
-
     if (writable > 0) {
         if ( writable > 1024*1000 ) //sanity check to prevent pa_xmalloc from crashing on monitor streams
             writable = 1024*1000;
         if (fbuffer_bytes > writable) {
-            if (quisk_sound_state.verbose_pulse)
-                printf("Truncating write by %u bytes\n", fbuffer_bytes - (int)writable);
+            if (quisk_sound_state.verbose_pulse && dev->dev_index <= t_MicPlayback)	// ignore for digital interfaces
+                printf("Truncating write on %s by %u bytes\n", dev->stream_description, fbuffer_bytes - (int)writable);
             fbuffer_bytes = writable;
         }
         pa_stream_write(dev->handle, fbuffer, (size_t)fbuffer_bytes, NULL, 0, PA_SEEK_RELATIVE);
-        //printf("wrote %d to %s\n", writable, dev->name);
     }
     else {
-        if (quisk_sound_state.verbose_pulse)
-            printf("Can't write to stream %s. Dropping %d bytes\n", dev->name, fbuffer_bytes);
+        if (quisk_sound_state.verbose_pulse && dev->dev_index <= t_MicPlayback)
+            printf("Can't write to stream %s. Dropping %d bytes\n", dev->stream_description, fbuffer_bytes);
     }
     
     pa_threaded_mainloop_unlock(pa_ml);
